!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ################
      MODULE MODD_DATA_TEB_n
!     ################
!
!!****  *MODD_DATA_TEB_n - declaration of surface parameters for urban surface
!!
!!    PURPOSE
!!    -------
!     Declaration of surface parameters
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       01/2004
!!      Modified       08/2012 G. Pigeon ROUGH_WALL, ROUGH_ROOF
!
!*       0.   DECLARATIONS
!             ------------
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DATA_TEB_t
!
  LOGICAL :: LDATA_BLDTYPE 
  LOGICAL :: LDATA_BLD_AGE
  LOGICAL :: LDATA_USETYPE 
  LOGICAL :: LDATA_GARDEN
  LOGICAL :: LDATA_GREENROOF
  LOGICAL :: LDATA_ROAD_DIR
  LOGICAL :: LDATA_BLD
  LOGICAL :: LDATA_BLD_HEIGHT
  LOGICAL :: LDATA_WALL_O_HOR
  LOGICAL :: LDATA_Z0_TOWN
  LOGICAL :: LDATA_ALB_ROOF
  LOGICAL :: LDATA_EMIS_ROOF
  LOGICAL :: LDATA_HC_ROOF
  LOGICAL :: LDATA_TC_ROOF
  LOGICAL :: LDATA_D_ROOF
  LOGICAL :: LDATA_ALB_ROAD
  LOGICAL :: LDATA_EMIS_ROAD
  LOGICAL :: LDATA_HC_ROAD
  LOGICAL :: LDATA_TC_ROAD
  LOGICAL :: LDATA_D_ROAD
  LOGICAL :: LDATA_ALB_WALL
  LOGICAL :: LDATA_EMIS_WALL
  LOGICAL :: LDATA_HC_WALL
  LOGICAL :: LDATA_TC_WALL
  LOGICAL :: LDATA_D_WALL
  LOGICAL :: LDATA_H_TRAFFIC
  LOGICAL :: LDATA_LE_TRAFFIC
  LOGICAL :: LDATA_LE_INDUSTRY
  LOGICAL :: LDATA_H_INDUSTRY
  LOGICAL :: LDATA_ROUGH_ROOF
  LOGICAL :: LDATA_ROUGH_WALL
!
! Number of layers in the specification of thermal characteristics
!
  INTEGER                    :: NPAR_ROOF_LAYER   ! number of layers in roofs
  INTEGER                    :: NPAR_ROAD_LAYER   ! number of layers in roads
  INTEGER                    :: NPAR_WALL_LAYER   ! number of layers in walls
!
!
! Geometric Parameters:
!
  INTEGER, POINTER, DIMENSION(:):: NPAR_BLDTYPE      ! type of buidlings
  INTEGER, POINTER, DIMENSION(:):: NPAR_BLD_AGE      ! date of construction of buildings
  INTEGER, POINTER, DIMENSION(:):: NPAR_BLDCODE      ! code for buildings (type+age)
  INTEGER, POINTER, DIMENSION(:):: NPAR_USETYPE      ! type of use in the buildings
  REAL, POINTER, DIMENSION(:)   :: XPAR_GARDEN       ! fraction of veg in the streets   (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_GREENROOF    ! fraction of greenroofs on roofs  (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_ROAD_DIR     ! road direction (° from North, clockwise)
  REAL, POINTER, DIMENSION(:)   :: XPAR_BLD          ! fraction of buildings            (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_BLD_HEIGHT   ! buildings height 'h'             (m)
  REAL, POINTER, DIMENSION(:)   :: XPAR_WALL_O_HOR   ! wall surf. / hor. surf.          (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_Z0_TOWN      ! roughness length for momentum    (m)
!
! Roof parameters
!
  REAL, POINTER, DIMENSION(:)   :: XPAR_ALB_ROOF     ! roof albedo                      (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_EMIS_ROOF    ! roof emissivity                  (-)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_HC_ROOF      ! roof layers heat capacity        (J/K/m3)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_TC_ROOF      ! roof layers thermal conductivity (W/K/m)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_D_ROOF       ! depth of roof layers             (m)
  REAL, POINTER, DIMENSION(:)   :: XPAR_ROUGH_ROOF   ! outside roof roughness coef 
!
!
! Road parameters
!
  REAL, POINTER, DIMENSION(:)   :: XPAR_ALB_ROAD     ! road albedo                      (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_EMIS_ROAD    ! road emissivity                  (-)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_HC_ROAD      ! road layers heat capacity        (J/K/m3)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_TC_ROAD      ! road layers thermal conductivity (W/K/m)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_D_ROAD       ! depth of road layers             (m)
!
! Wall parameters
!
  REAL, POINTER, DIMENSION(:)   :: XPAR_ALB_WALL     ! wall albedo                      (-)
  REAL, POINTER, DIMENSION(:)   :: XPAR_EMIS_WALL    ! wall emissivity                  (-)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_HC_WALL      ! wall layers heat capacity        (J/K/m3)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_TC_WALL      ! wall layers thermal conductivity (W/K/m)
  REAL, POINTER, DIMENSION(:,:) :: XPAR_D_WALL       ! depth of wall layers             (m)
  REAL, POINTER, DIMENSION(:)   :: XPAR_ROUGH_WALL   ! outside wall roughness coef
!
! anthropogenic fluxes
!
  REAL, POINTER, DIMENSION(:)   :: XPAR_H_TRAFFIC    ! anthropogenic sensible
!                                                  ! heat fluxes due to traffic       (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XPAR_LE_TRAFFIC   ! anthropogenic latent
!                                                  ! heat fluxes due to traffic       (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XPAR_H_INDUSTRY   ! anthropogenic sensible                   
!                                                  ! heat fluxes due to factories     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XPAR_LE_INDUSTRY  ! anthropogenic latent
!                                                  ! heat fluxes due to factories     (W/m2)
!
END TYPE DATA_TEB_t

TYPE(DATA_TEB_t), ALLOCATABLE, TARGET, SAVE :: DATA_TEB_MODEL(:)

LOGICAL, POINTER :: LDATA_BLDTYPE=>NULL()
!$OMP THREADPRIVATE(LDATA_BLDTYPE)
LOGICAL, POINTER :: LDATA_BLD_AGE=>NULL()
!$OMP THREADPRIVATE(LDATA_BLD_AGE)
LOGICAL, POINTER :: LDATA_BLDCODE=>NULL()
!$OMP THREADPRIVATE(LDATA_BLDCODE)
LOGICAL, POINTER :: LDATA_USETYPE=>NULL()
!$OMP THREADPRIVATE(LDATA_USETYPE)
LOGICAL, POINTER :: LDATA_GARDEN=>NULL()
!$OMP THREADPRIVATE(LDATA_GARDEN)
LOGICAL, POINTER :: LDATA_GREENROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_GREENROOF)
LOGICAL, POINTER :: LDATA_ROAD_DIR=>NULL()
!$OMP THREADPRIVATE(LDATA_ROAD_DIR)
LOGICAL, POINTER :: LDATA_BLD=>NULL()
!$OMP THREADPRIVATE(LDATA_BLD)
LOGICAL, POINTER :: LDATA_BLD_HEIGHT=>NULL()
!$OMP THREADPRIVATE(LDATA_BLD_HEIGHT)
LOGICAL, POINTER :: LDATA_WALL_O_HOR=>NULL()
!$OMP THREADPRIVATE(LDATA_WALL_O_HOR)
LOGICAL, POINTER :: LDATA_Z0_TOWN=>NULL()
!$OMP THREADPRIVATE(LDATA_Z0_TOWN)
LOGICAL, POINTER :: LDATA_ALB_ROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_ALB_ROOF)
LOGICAL, POINTER :: LDATA_EMIS_ROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_EMIS_ROOF)
LOGICAL, POINTER :: LDATA_HC_ROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_HC_ROOF)
LOGICAL, POINTER :: LDATA_TC_ROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_TC_ROOF)
LOGICAL, POINTER :: LDATA_D_ROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_D_ROOF)
LOGICAL, POINTER :: LDATA_ALB_ROAD=>NULL()
!$OMP THREADPRIVATE(LDATA_ALB_ROAD)
LOGICAL, POINTER :: LDATA_EMIS_ROAD=>NULL()
!$OMP THREADPRIVATE(LDATA_EMIS_ROAD)
LOGICAL, POINTER :: LDATA_HC_ROAD=>NULL()
!$OMP THREADPRIVATE(LDATA_HC_ROAD)
LOGICAL, POINTER :: LDATA_TC_ROAD=>NULL()
!$OMP THREADPRIVATE(LDATA_TC_ROAD)
LOGICAL, POINTER :: LDATA_D_ROAD=>NULL()
!$OMP THREADPRIVATE(LDATA_D_ROAD)
LOGICAL, POINTER :: LDATA_ALB_WALL=>NULL()
!$OMP THREADPRIVATE(LDATA_ALB_WALL)
LOGICAL, POINTER :: LDATA_EMIS_WALL=>NULL()
!$OMP THREADPRIVATE(LDATA_EMIS_WALL)
LOGICAL, POINTER :: LDATA_HC_WALL=>NULL()
!$OMP THREADPRIVATE(LDATA_HC_WALL)
LOGICAL, POINTER :: LDATA_TC_WALL=>NULL()
!$OMP THREADPRIVATE(LDATA_TC_WALL)
LOGICAL, POINTER :: LDATA_D_WALL=>NULL()
!$OMP THREADPRIVATE(LDATA_D_WALL)
LOGICAL, POINTER :: LDATA_H_TRAFFIC=>NULL()
!$OMP THREADPRIVATE(LDATA_H_TRAFFIC)
LOGICAL, POINTER :: LDATA_LE_TRAFFIC=>NULL()
!$OMP THREADPRIVATE(LDATA_LE_TRAFFIC)
LOGICAL, POINTER :: LDATA_LE_INDUSTRY=>NULL()
!$OMP THREADPRIVATE(LDATA_LE_INDUSTRY)
LOGICAL, POINTER :: LDATA_H_INDUSTRY=>NULL()
!$OMP THREADPRIVATE(LDATA_H_INDUSTRY)
LOGICAL, POINTER :: LDATA_ROUGH_ROOF=>NULL()
!$OMP THREADPRIVATE(LDATA_ROUGH_ROOF)
LOGICAL, POINTER :: LDATA_ROUGH_WALL=>NULL()
!$OMP THREADPRIVATE(LDATA_ROUGH_WALL)
INTEGER, POINTER :: NPAR_ROOF_LAYER=>NULL()
!$OMP THREADPRIVATE(NPAR_ROOF_LAYER)
INTEGER, POINTER :: NPAR_ROAD_LAYER=>NULL()
!$OMP THREADPRIVATE(NPAR_ROAD_LAYER)
INTEGER, POINTER :: NPAR_WALL_LAYER=>NULL()
!$OMP THREADPRIVATE(NPAR_WALL_LAYER)
INTEGER, POINTER, DIMENSION(:)   :: NPAR_BLDTYPE=>NULL()
!$OMP THREADPRIVATE(NPAR_BLDTYPE)
INTEGER, POINTER, DIMENSION(:)   :: NPAR_BLD_AGE=>NULL()
!$OMP THREADPRIVATE(NPAR_BLD_AGE)
INTEGER, POINTER, DIMENSION(:)   :: NPAR_BLDCODE=>NULL()
!$OMP THREADPRIVATE(NPAR_BLDCODE)
INTEGER, POINTER, DIMENSION(:)   :: NPAR_USETYPE=>NULL()
!$OMP THREADPRIVATE(NPAR_USETYPE)
REAL, POINTER, DIMENSION(:)   :: XPAR_GARDEN=>NULL()
!$OMP THREADPRIVATE(XPAR_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XPAR_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XPAR_ROAD_DIR=>NULL()
!$OMP THREADPRIVATE(XPAR_ROAD_DIR)
REAL, POINTER, DIMENSION(:)   :: XPAR_BLD=>NULL()
!$OMP THREADPRIVATE(XPAR_BLD)
REAL, POINTER, DIMENSION(:)   :: XPAR_BLD_HEIGHT=>NULL()
!$OMP THREADPRIVATE(XPAR_BLD_HEIGHT)
REAL, POINTER, DIMENSION(:)   :: XPAR_WALL_O_HOR=>NULL()
!$OMP THREADPRIVATE(XPAR_WALL_O_HOR)
REAL, POINTER, DIMENSION(:)   :: XPAR_Z0_TOWN=>NULL()
!$OMP THREADPRIVATE(XPAR_Z0_TOWN)
REAL, POINTER, DIMENSION(:)   :: XPAR_ALB_ROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_ALB_ROOF)
REAL, POINTER, DIMENSION(:)   :: XPAR_EMIS_ROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_EMIS_ROOF)
REAL, POINTER, DIMENSION(:,:) :: XPAR_HC_ROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_HC_ROOF)
REAL, POINTER, DIMENSION(:,:) :: XPAR_TC_ROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_TC_ROOF)
REAL, POINTER, DIMENSION(:,:) :: XPAR_D_ROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_D_ROOF)
REAL, POINTER, DIMENSION(:)   :: XPAR_ALB_ROAD=>NULL()
!$OMP THREADPRIVATE(XPAR_ALB_ROAD)
REAL, POINTER, DIMENSION(:)   :: XPAR_EMIS_ROAD=>NULL()
!$OMP THREADPRIVATE(XPAR_EMIS_ROAD)
REAL, POINTER, DIMENSION(:,:) :: XPAR_HC_ROAD=>NULL()
!$OMP THREADPRIVATE(XPAR_HC_ROAD)
REAL, POINTER, DIMENSION(:,:) :: XPAR_TC_ROAD=>NULL()
!$OMP THREADPRIVATE(XPAR_TC_ROAD)
REAL, POINTER, DIMENSION(:,:) :: XPAR_D_ROAD=>NULL()
!$OMP THREADPRIVATE(XPAR_D_ROAD)
REAL, POINTER, DIMENSION(:)   :: XPAR_ALB_WALL=>NULL()
!$OMP THREADPRIVATE(XPAR_ALB_WALL)
REAL, POINTER, DIMENSION(:)   :: XPAR_EMIS_WALL=>NULL()
!$OMP THREADPRIVATE(XPAR_EMIS_WALL)
REAL, POINTER, DIMENSION(:,:) :: XPAR_HC_WALL=>NULL()
!$OMP THREADPRIVATE(XPAR_HC_WALL)
REAL, POINTER, DIMENSION(:,:) :: XPAR_TC_WALL=>NULL()
!$OMP THREADPRIVATE(XPAR_TC_WALL)
REAL, POINTER, DIMENSION(:,:) :: XPAR_D_WALL=>NULL()
!$OMP THREADPRIVATE(XPAR_D_WALL)
REAL, POINTER, DIMENSION(:)   :: XPAR_H_TRAFFIC=>NULL()
!$OMP THREADPRIVATE(XPAR_H_TRAFFIC)
REAL, POINTER, DIMENSION(:)   :: XPAR_LE_TRAFFIC=>NULL()
!$OMP THREADPRIVATE(XPAR_LE_TRAFFIC)
REAL, POINTER, DIMENSION(:)   :: XPAR_H_INDUSTRY=>NULL()
!$OMP THREADPRIVATE(XPAR_H_INDUSTRY)
REAL, POINTER, DIMENSION(:)   :: XPAR_LE_INDUSTRY=>NULL()
!$OMP THREADPRIVATE(XPAR_LE_INDUSTRY)
REAL, POINTER, DIMENSION(:)   :: XPAR_ROUGH_ROOF=>NULL()
!$OMP THREADPRIVATE(XPAR_ROUGH_ROOF)
REAL, POINTER, DIMENSION(:)   :: XPAR_ROUGH_WALL=>NULL()
!$OMP THREADPRIVATE(XPAR_ROUGH_WALL)

CONTAINS

SUBROUTINE DATA_TEB_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DATA_TEB_MODEL(KFROM)%NPAR_BLDTYPE=>NPAR_BLDTYPE
DATA_TEB_MODEL(KFROM)%NPAR_BLD_AGE=>NPAR_BLD_AGE
DATA_TEB_MODEL(KFROM)%NPAR_BLDCODE=>NPAR_BLDCODE
DATA_TEB_MODEL(KFROM)%NPAR_USETYPE=>NPAR_USETYPE
DATA_TEB_MODEL(KFROM)%XPAR_GARDEN=>XPAR_GARDEN
DATA_TEB_MODEL(KFROM)%XPAR_GREENROOF=>XPAR_GREENROOF
DATA_TEB_MODEL(KFROM)%XPAR_ROAD_DIR=>XPAR_ROAD_DIR
DATA_TEB_MODEL(KFROM)%XPAR_BLD=>XPAR_BLD
DATA_TEB_MODEL(KFROM)%XPAR_BLD_HEIGHT=>XPAR_BLD_HEIGHT
DATA_TEB_MODEL(KFROM)%XPAR_WALL_O_HOR=>XPAR_WALL_O_HOR
DATA_TEB_MODEL(KFROM)%XPAR_Z0_TOWN=>XPAR_Z0_TOWN
DATA_TEB_MODEL(KFROM)%XPAR_ALB_ROOF=>XPAR_ALB_ROOF
DATA_TEB_MODEL(KFROM)%XPAR_EMIS_ROOF=>XPAR_EMIS_ROOF
DATA_TEB_MODEL(KFROM)%XPAR_HC_ROOF=>XPAR_HC_ROOF
DATA_TEB_MODEL(KFROM)%XPAR_TC_ROOF=>XPAR_TC_ROOF
DATA_TEB_MODEL(KFROM)%XPAR_D_ROOF=>XPAR_D_ROOF
DATA_TEB_MODEL(KFROM)%XPAR_ALB_ROAD=>XPAR_ALB_ROAD
DATA_TEB_MODEL(KFROM)%XPAR_EMIS_ROAD=>XPAR_EMIS_ROAD
DATA_TEB_MODEL(KFROM)%XPAR_HC_ROAD=>XPAR_HC_ROAD
DATA_TEB_MODEL(KFROM)%XPAR_TC_ROAD=>XPAR_TC_ROAD
DATA_TEB_MODEL(KFROM)%XPAR_D_ROAD=>XPAR_D_ROAD
DATA_TEB_MODEL(KFROM)%XPAR_ALB_WALL=>XPAR_ALB_WALL
DATA_TEB_MODEL(KFROM)%XPAR_EMIS_WALL=>XPAR_EMIS_WALL
DATA_TEB_MODEL(KFROM)%XPAR_HC_WALL=>XPAR_HC_WALL
DATA_TEB_MODEL(KFROM)%XPAR_TC_WALL=>XPAR_TC_WALL
DATA_TEB_MODEL(KFROM)%XPAR_D_WALL=>XPAR_D_WALL
DATA_TEB_MODEL(KFROM)%XPAR_H_TRAFFIC=>XPAR_H_TRAFFIC
DATA_TEB_MODEL(KFROM)%XPAR_LE_TRAFFIC=>XPAR_LE_TRAFFIC
DATA_TEB_MODEL(KFROM)%XPAR_H_INDUSTRY=>XPAR_H_INDUSTRY
DATA_TEB_MODEL(KFROM)%XPAR_LE_INDUSTRY=>XPAR_LE_INDUSTRY
DATA_TEB_MODEL(KFROM)%XPAR_ROUGH_ROOF=>XPAR_ROUGH_ROOF
DATA_TEB_MODEL(KFROM)%XPAR_ROUGH_WALL=>XPAR_ROUGH_WALL
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DATA_TEB_N:DATA_TEB_GOTO_MODEL',0,ZHOOK_HANDLE)
LDATA_BLDTYPE=>DATA_TEB_MODEL(KTO)%LDATA_BLDTYPE
LDATA_BLD_AGE=>DATA_TEB_MODEL(KTO)%LDATA_BLD_AGE
LDATA_USETYPE=>DATA_TEB_MODEL(KTO)%LDATA_USETYPE
LDATA_GARDEN=>DATA_TEB_MODEL(KTO)%LDATA_GARDEN
LDATA_GREENROOF=>DATA_TEB_MODEL(KTO)%LDATA_GREENROOF
LDATA_ROAD_DIR=>DATA_TEB_MODEL(KTO)%LDATA_ROAD_DIR
LDATA_BLD=>DATA_TEB_MODEL(KTO)%LDATA_BLD
LDATA_BLD_HEIGHT=>DATA_TEB_MODEL(KTO)%LDATA_BLD_HEIGHT
LDATA_WALL_O_HOR=>DATA_TEB_MODEL(KTO)%LDATA_WALL_O_HOR
LDATA_Z0_TOWN=>DATA_TEB_MODEL(KTO)%LDATA_Z0_TOWN
LDATA_ALB_ROOF=>DATA_TEB_MODEL(KTO)%LDATA_ALB_ROOF
LDATA_EMIS_ROOF=>DATA_TEB_MODEL(KTO)%LDATA_EMIS_ROOF
LDATA_HC_ROOF=>DATA_TEB_MODEL(KTO)%LDATA_HC_ROOF
LDATA_TC_ROOF=>DATA_TEB_MODEL(KTO)%LDATA_TC_ROOF
LDATA_D_ROOF=>DATA_TEB_MODEL(KTO)%LDATA_D_ROOF
LDATA_ALB_ROAD=>DATA_TEB_MODEL(KTO)%LDATA_ALB_ROAD
LDATA_EMIS_ROAD=>DATA_TEB_MODEL(KTO)%LDATA_EMIS_ROAD
LDATA_HC_ROAD=>DATA_TEB_MODEL(KTO)%LDATA_HC_ROAD
LDATA_TC_ROAD=>DATA_TEB_MODEL(KTO)%LDATA_TC_ROAD
LDATA_D_ROAD=>DATA_TEB_MODEL(KTO)%LDATA_D_ROAD
LDATA_ALB_WALL=>DATA_TEB_MODEL(KTO)%LDATA_ALB_WALL
LDATA_EMIS_WALL=>DATA_TEB_MODEL(KTO)%LDATA_EMIS_WALL
LDATA_HC_WALL=>DATA_TEB_MODEL(KTO)%LDATA_HC_WALL
LDATA_TC_WALL=>DATA_TEB_MODEL(KTO)%LDATA_TC_WALL
LDATA_D_WALL=>DATA_TEB_MODEL(KTO)%LDATA_D_WALL
LDATA_H_TRAFFIC=>DATA_TEB_MODEL(KTO)%LDATA_H_TRAFFIC
LDATA_LE_TRAFFIC=>DATA_TEB_MODEL(KTO)%LDATA_LE_TRAFFIC
LDATA_LE_INDUSTRY=>DATA_TEB_MODEL(KTO)%LDATA_LE_INDUSTRY
LDATA_H_INDUSTRY=>DATA_TEB_MODEL(KTO)%LDATA_H_INDUSTRY
LDATA_ROUGH_ROOF=>DATA_TEB_MODEL(KTO)%LDATA_ROUGH_ROOF
LDATA_ROUGH_WALL=>DATA_TEB_MODEL(KTO)%LDATA_ROUGH_WALL

NPAR_ROOF_LAYER=>DATA_TEB_MODEL(KTO)%NPAR_ROOF_LAYER
NPAR_ROAD_LAYER=>DATA_TEB_MODEL(KTO)%NPAR_ROAD_LAYER
NPAR_WALL_LAYER=>DATA_TEB_MODEL(KTO)%NPAR_WALL_LAYER
NPAR_BLDTYPE=>DATA_TEB_MODEL(KTO)%NPAR_BLDTYPE
NPAR_BLD_AGE=>DATA_TEB_MODEL(KTO)%NPAR_BLD_AGE
NPAR_BLDCODE=>DATA_TEB_MODEL(KTO)%NPAR_BLDCODE
NPAR_USETYPE=>DATA_TEB_MODEL(KTO)%NPAR_USETYPE
XPAR_GARDEN=>DATA_TEB_MODEL(KTO)%XPAR_GARDEN
XPAR_GREENROOF=>DATA_TEB_MODEL(KTO)%XPAR_GREENROOF
XPAR_ROAD_DIR=>DATA_TEB_MODEL(KTO)%XPAR_ROAD_DIR
XPAR_BLD=>DATA_TEB_MODEL(KTO)%XPAR_BLD
XPAR_BLD_HEIGHT=>DATA_TEB_MODEL(KTO)%XPAR_BLD_HEIGHT
XPAR_WALL_O_HOR=>DATA_TEB_MODEL(KTO)%XPAR_WALL_O_HOR
XPAR_Z0_TOWN=>DATA_TEB_MODEL(KTO)%XPAR_Z0_TOWN
XPAR_ALB_ROOF=>DATA_TEB_MODEL(KTO)%XPAR_ALB_ROOF
XPAR_EMIS_ROOF=>DATA_TEB_MODEL(KTO)%XPAR_EMIS_ROOF
XPAR_HC_ROOF=>DATA_TEB_MODEL(KTO)%XPAR_HC_ROOF
XPAR_TC_ROOF=>DATA_TEB_MODEL(KTO)%XPAR_TC_ROOF
XPAR_D_ROOF=>DATA_TEB_MODEL(KTO)%XPAR_D_ROOF
XPAR_ALB_ROAD=>DATA_TEB_MODEL(KTO)%XPAR_ALB_ROAD
XPAR_EMIS_ROAD=>DATA_TEB_MODEL(KTO)%XPAR_EMIS_ROAD
XPAR_HC_ROAD=>DATA_TEB_MODEL(KTO)%XPAR_HC_ROAD
XPAR_TC_ROAD=>DATA_TEB_MODEL(KTO)%XPAR_TC_ROAD
XPAR_D_ROAD=>DATA_TEB_MODEL(KTO)%XPAR_D_ROAD
XPAR_ALB_WALL=>DATA_TEB_MODEL(KTO)%XPAR_ALB_WALL
XPAR_EMIS_WALL=>DATA_TEB_MODEL(KTO)%XPAR_EMIS_WALL
XPAR_HC_WALL=>DATA_TEB_MODEL(KTO)%XPAR_HC_WALL
XPAR_TC_WALL=>DATA_TEB_MODEL(KTO)%XPAR_TC_WALL
XPAR_D_WALL=>DATA_TEB_MODEL(KTO)%XPAR_D_WALL
XPAR_H_TRAFFIC=>DATA_TEB_MODEL(KTO)%XPAR_H_TRAFFIC
XPAR_LE_TRAFFIC=>DATA_TEB_MODEL(KTO)%XPAR_LE_TRAFFIC
XPAR_H_INDUSTRY=>DATA_TEB_MODEL(KTO)%XPAR_H_INDUSTRY
XPAR_LE_INDUSTRY=>DATA_TEB_MODEL(KTO)%XPAR_LE_INDUSTRY
XPAR_ROUGH_ROOF=>DATA_TEB_MODEL(KTO)%XPAR_ROUGH_ROOF
XPAR_ROUGH_WALL=>DATA_TEB_MODEL(KTO)%XPAR_ROUGH_WALL
IF (LHOOK) CALL DR_HOOK('MODD_DATA_TEB_N:DATA_TEB_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DATA_TEB_GOTO_MODEL

SUBROUTINE DATA_TEB_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_N:DATA_TEB_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DATA_TEB_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DATA_TEB_MODEL(J)%NPAR_BLDTYPE)
  NULLIFY(DATA_TEB_MODEL(J)%NPAR_BLD_AGE)
  NULLIFY(DATA_TEB_MODEL(J)%NPAR_BLDCODE)
  NULLIFY(DATA_TEB_MODEL(J)%NPAR_USETYPE)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_GARDEN)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_GREENROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_ROAD_DIR)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_BLD)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_BLD_HEIGHT)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_WALL_O_HOR)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_Z0_TOWN)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_ALB_ROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_EMIS_ROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_HC_ROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_TC_ROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_D_ROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_ALB_ROAD)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_EMIS_ROAD)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_HC_ROAD)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_TC_ROAD)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_D_ROAD)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_ALB_WALL)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_EMIS_WALL)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_HC_WALL)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_TC_WALL)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_D_WALL)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_H_TRAFFIC)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_LE_TRAFFIC)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_H_INDUSTRY)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_LE_INDUSTRY)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_ROUGH_ROOF)
  NULLIFY(DATA_TEB_MODEL(J)%XPAR_ROUGH_WALL)
ENDDO
DATA_TEB_MODEL(:)%LDATA_BLDTYPE=.FALSE.
DATA_TEB_MODEL(:)%LDATA_BLD_AGE=.FALSE.
DATA_TEB_MODEL(:)%LDATA_USETYPE=.FALSE.
DATA_TEB_MODEL(:)%LDATA_GARDEN=.FALSE.
DATA_TEB_MODEL(:)%LDATA_GREENROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_ROAD_DIR=.FALSE.
DATA_TEB_MODEL(:)%LDATA_BLD=.FALSE.
DATA_TEB_MODEL(:)%LDATA_BLD_HEIGHT=.FALSE.
DATA_TEB_MODEL(:)%LDATA_WALL_O_HOR=.FALSE.
DATA_TEB_MODEL(:)%LDATA_Z0_TOWN=.FALSE.
DATA_TEB_MODEL(:)%LDATA_ALB_ROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_EMIS_ROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_HC_ROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_TC_ROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_D_ROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_ALB_ROAD=.FALSE.
DATA_TEB_MODEL(:)%LDATA_EMIS_ROAD=.FALSE.
DATA_TEB_MODEL(:)%LDATA_HC_ROAD=.FALSE.
DATA_TEB_MODEL(:)%LDATA_TC_ROAD=.FALSE.
DATA_TEB_MODEL(:)%LDATA_D_ROAD=.FALSE.
DATA_TEB_MODEL(:)%LDATA_ALB_WALL=.FALSE.
DATA_TEB_MODEL(:)%LDATA_EMIS_WALL=.FALSE.
DATA_TEB_MODEL(:)%LDATA_HC_WALL=.FALSE.
DATA_TEB_MODEL(:)%LDATA_TC_WALL=.FALSE.
DATA_TEB_MODEL(:)%LDATA_D_WALL=.FALSE.
DATA_TEB_MODEL(:)%LDATA_H_TRAFFIC=.FALSE.
DATA_TEB_MODEL(:)%LDATA_LE_TRAFFIC=.FALSE.
DATA_TEB_MODEL(:)%LDATA_H_INDUSTRY=.FALSE.
DATA_TEB_MODEL(:)%LDATA_LE_INDUSTRY=.FALSE.
DATA_TEB_MODEL(:)%LDATA_ROUGH_ROOF=.FALSE.
DATA_TEB_MODEL(:)%LDATA_ROUGH_WALL=.FALSE.
DATA_TEB_MODEL(:)%NPAR_ROOF_LAYER=0
DATA_TEB_MODEL(:)%NPAR_ROAD_LAYER=0
DATA_TEB_MODEL(:)%NPAR_WALL_LAYER=0
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_N:DATA_TEB_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DATA_TEB_ALLOC

SUBROUTINE DATA_TEB_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_N:DATA_TEB_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DATA_TEB_MODEL)) DEALLOCATE(DATA_TEB_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DATA_TEB_N:DATA_TEB_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DATA_TEB_DEALLO

END MODULE MODD_DATA_TEB_n
