!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ############################
      MODULE MODD_DIAG_MISC_TEB_n
!     ############################
!
!!****  *MODD_DIAG_MISC_TEB - declaration of packed surface parameters for TEB scheme
!!
!!    PURPOSE
!!    -------
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!
!!    AUTHOR
!!    ------
!!	P. Le Moigne   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original       07/10/04
!
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DIAG_MISC_TEB_OPTIONS_t
!------------------------------------------------------------------------------
!
  LOGICAL :: LSURF_MISC_BUDGET   ! flag for miscellaneous terms of teb scheme
  LOGICAL :: LSURF_EVAP_BUDGET   ! flag for all terms of evaporation
  LOGICAL :: LSURF_DIAG_ALBEDO   ! flag to write out diagnostic albedo  
  REAL, POINTER, DIMENSION(:)   :: XUTCI_IN      ! UTCI for person indoor
  REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSUN  ! UTCI for person outdoor at sun
  REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSHADE! UTCI for person outdoor at shade
  REAL, POINTER, DIMENSION(:)   :: XTRAD_SUN! Mean radiant temperature seen by person at sun (K)
  REAL, POINTER, DIMENSION(:)   :: XTRAD_SHADE! Mean radiant temperature seen by person in shade (K)
!
END TYPE DIAG_MISC_TEB_OPTIONS_t
!
TYPE DIAG_MISC_TEB_t
!
!* miscellaneous variables
!
  REAL, POINTER, DIMENSION(:)   :: XZ0_TOWN  ! town roughness length
  REAL, POINTER, DIMENSION(:)   :: XQF_BLD   ! domestic heating
  REAL, POINTER, DIMENSION(:)   :: XFLX_BLD ! heat flux from bld
  REAL, POINTER, DIMENSION(:)   :: XQF_TOWN  ! total anthropogenic heat
  REAL, POINTER, DIMENSION(:)   :: XDQS_TOWN ! storage inside building
!
  REAL, POINTER, DIMENSION(:)   :: XH_WALL_A   ! wall sensible heat flux          (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_WALL_B   ! wall sensible heat flux          (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_ROOF     ! roof sensible heat flux          (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_ROAD     ! road sensible heat flux          (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_GARDEN   ! green area sensible heat flux    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_GREENROOF! green roof sensible heat flux    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_STRLROOF ! structural roof sens. heat flux  (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XH_BLT      ! built surf sensible heat flux    (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_WALL_A  ! net radiation at wall            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_WALL_B  ! net radiation at wall            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_ROOF    ! net radiation at roof            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_ROAD    ! net radiation at road            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_GARDEN  ! net radiation at green areas     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_GREENROOF!net radiation at green roofs     (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_STRLROOF !net radiation at structural roofs(W/m2)
  REAL, POINTER, DIMENSION(:)   :: XRN_BLT     ! net radiation at built surf      (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_WALL_A !net wall conduction flux        (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_WALL_B !net wall conduction flux        (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_ROOF ! net roof conduction flux         (W/m2)                                         
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_ROAD ! net road conduction flux         (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_GARDEN!net green area conduction flux   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_GREENROOF!net green roof conduction flux(W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_STRLROOF !net structural roof cond flux (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XGFLUX_BLT  ! net built surf conduction flux   (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE_ROOF    ! roof latent heat flux            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE_ROAD    ! road latent heat flux            (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE_GARDEN  ! green area latent heat flux      (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE_GREENROOF!green roof latent heat flux      (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE_STRLROOF !structural roof latent heat flux (W/m2)
  REAL, POINTER, DIMENSION(:)   :: XLE_BLT     ! built surf latent heat flux      (W/m2)
!
  REAL, POINTER, DIMENSION(:)   :: XRUNOFF_GREENROOF ! water runoff for greenroof (kg/m2/s)
  REAL, POINTER, DIMENSION(:)   :: XDRAIN_GREENROOF  ! water vertical drainage for greenroof (kg/m2/s)
!
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_ROOF      ! Downward shortwave radiation over roofs
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_SNOW_ROOF ! Downward longwave radiation over roofs
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_ROOF      ! Upward shortwave radiation over roofs
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_SNOW_ROOF ! Upward longwave radiation over roofs
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_ROAD      ! Downward shortwave radiation over roads
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_SNOW_ROAD ! Downward longwave radiation over roads
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_ROAD      ! Upward shortwave radiation over roads
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_SNOW_ROAD ! Upward longwave radiation over roads
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_WALL_A    ! Downward shortwave radiation over walls
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_WALL_B    ! Downward shortwave radiation over walls
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_WALL_A    ! Upward shortwave radiation over walls
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_WALL_B    ! Upward shortwave radiation over walls
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_GARDEN    ! Downward shortwave radiation over green areas
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_GARDEN    ! Upward shortwave radiation over green areas
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_GREENROOF ! Downward shortwave radiation over green roofs
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_GREENROOF ! Upward shortwave radiation over green roofs
  REAL, POINTER, DIMENSION(:)   :: XG_GREENROOF_ROOF ! Heat flux between green roof and structural roof
!
  REAL, POINTER, DIMENSION(:)   :: XH_BLD_COOL       ! Sensible cooling energy demand  
                                                     ! of the building [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XT_BLD_COOL       ! Total cooling energy demand  
                                                     ! of the building [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XH_BLD_HEAT       ! Heating energy demand       
                                                     ! of the building [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XLE_BLD_COOL      ! Latent cooling energy demand 
                                                     ! of the building [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XLE_BLD_HEAT      ! Latent heating energy demand 
                                                     ! of the building [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XH_WASTE          ! Sensible waste heat from HVAC system
                                                     ! [W m-2(tot)]
  REAL, POINTER, DIMENSION(:)   :: XLE_WASTE         ! Latent waste heat from HVAC system
                                                     ! [W m-2(tot)]
  REAL, POINTER, DIMENSION(:)   :: XHVAC_COOL        ! Energy consumption of the cooling system
                                                     ! [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XHVAC_HEAT        ! Energy consumption of the heating system
                                                     ! [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XCAP_SYS          ! Actual capacity of the cooling system
                                                     ! [W m-2(bld)] 
  REAL, POINTER, DIMENSION(:)   :: XM_SYS            ! Actual HVAC mass flow rate 
                                                     ! [kg s-1 m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XCOP              ! COP of the cooling system
  REAL, POINTER, DIMENSION(:)   :: XQ_SYS            ! Supply air specific humidity [kg kg-1]
  REAL, POINTER, DIMENSION(:)   :: XT_SYS            ! Supply air temperature [K]
  REAL, POINTER, DIMENSION(:)   :: XTR_SW_WIN        ! Solar radiation transmitted throught
                                                     ! windows [W m-2(bld)]
  REAL, POINTER, DIMENSION(:)   :: XFAN_POWER        ! HVAC fan power
  REAL, POINTER, DIMENSION(:)   :: XABS_SW_WIN       ! window absorbed shortwave radiation [W m-2] 
  REAL, POINTER, DIMENSION(:)   :: XABS_LW_WIN       ! absorbed infrared rad. [W m-2]

  REAL, POINTER, DIMENSION(:)   :: XEMIT_LW_FAC      ! LW flux emitted by the facade (W/m² facade)
  REAL, POINTER, DIMENSION(:)   :: XEMIT_LW_GRND     ! LW flux emitted by the ground (W/m² ground = road + garden)
  REAL, POINTER, DIMENSION(:)   :: XT_RAD_IND     !Indoor mean radiant temperature [K]
  REAL, POINTER, DIMENSION(:)   :: XREF_SW_GRND ! total solar rad reflected by ground
  REAL, POINTER, DIMENSION(:)   :: XREF_SW_FAC ! total solar rad reflected by facade
  REAL, POINTER, DIMENSION(:)   :: XHU_BLD !Indoor relative humidity
!
!------------------------------------------------------------------------------
!

END TYPE DIAG_MISC_TEB_t

TYPE(DIAG_MISC_TEB_OPTIONS_t), ALLOCATABLE, TARGET, SAVE :: DIAG_MISC_TEB_OPTIONS_MODEL(:)
TYPE(DIAG_MISC_TEB_t), ALLOCATABLE, TARGET, SAVE :: DIAG_MISC_TEB_MODEL(:,:)

LOGICAL, POINTER :: LSURF_MISC_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_MISC_BUDGET)
LOGICAL, POINTER :: LSURF_EVAP_BUDGET=>NULL()
!$OMP THREADPRIVATE(LSURF_EVAP_BUDGET)
LOGICAL, POINTER :: LSURF_DIAG_ALBEDO=>NULL()
!$OMP THREADPRIVATE(LSURF_DIAG_ALBEDO)
REAL, POINTER, DIMENSION(:)   :: XZ0_TOWN=>NULL()
!$OMP THREADPRIVATE(XZ0_TOWN)
REAL, POINTER, DIMENSION(:)   :: XQF_BLD=>NULL()
!$OMP THREADPRIVATE(XQF_BLD)
REAL, POINTER, DIMENSION(:)   :: XFLX_BLD=>NULL()
!$OMP THREADPRIVATE(XFLX_BLD)
REAL, POINTER, DIMENSION(:)   :: XQF_TOWN=>NULL()
!$OMP THREADPRIVATE(XQF_TOWN)
REAL, POINTER, DIMENSION(:)   :: XDQS_TOWN=>NULL()
!$OMP THREADPRIVATE(XDQS_TOWN)
REAL, POINTER, DIMENSION(:)   :: XH_WALL_A=>NULL()
!$OMP THREADPRIVATE(XH_WALL_A)
REAL, POINTER, DIMENSION(:)   :: XH_WALL_B=>NULL()
!$OMP THREADPRIVATE(XH_WALL_B)
REAL, POINTER, DIMENSION(:)   :: XH_ROOF=>NULL()
!$OMP THREADPRIVATE(XH_ROOF)
REAL, POINTER, DIMENSION(:)   :: XH_ROAD=>NULL()
!$OMP THREADPRIVATE(XH_ROAD)
REAL, POINTER, DIMENSION(:)   :: XRN_WALL_A=>NULL()
!$OMP THREADPRIVATE(XRN_WALL_A)
REAL, POINTER, DIMENSION(:)   :: XRN_WALL_B=>NULL()
!$OMP THREADPRIVATE(XRN_WALL_B)
REAL, POINTER, DIMENSION(:)   :: XRN_ROOF=>NULL()
!$OMP THREADPRIVATE(XRN_ROOF)
REAL, POINTER, DIMENSION(:)   :: XRN_ROAD=>NULL()
!$OMP THREADPRIVATE(XRN_ROAD)
REAL, POINTER, DIMENSION(:)   :: XRN_GARDEN=>NULL()
!$OMP THREADPRIVATE(XRN_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XRN_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XRN_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XRN_STRLROOF=>NULL()
!$OMP THREADPRIVATE(XRN_STRLROOF)
REAL, POINTER, DIMENSION(:)   :: XRN_BLT=>NULL()
!$OMP THREADPRIVATE(XRN_BLT)
REAL, POINTER, DIMENSION(:)   :: XH_GARDEN=>NULL()
!$OMP THREADPRIVATE(XH_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XH_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XH_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XH_STRLROOF=>NULL()
!$OMP THREADPRIVATE(XH_STRLROOF)
REAL, POINTER, DIMENSION(:)   :: XH_BLT=>NULL()
!$OMP THREADPRIVATE(XH_BLT)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_WALL_A=>NULL()
!$OMP THREADPRIVATE(XGFLUX_WALL_A)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_WALL_B=>NULL()
!$OMP THREADPRIVATE(XGFLUX_WALL_B)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_ROOF=>NULL()
!$OMP THREADPRIVATE(XGFLUX_ROOF)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_ROAD=>NULL()
!$OMP THREADPRIVATE(XGFLUX_ROAD)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_GARDEN=>NULL()
!$OMP THREADPRIVATE(XGFLUX_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XGFLUX_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_STRLROOF=>NULL()
!$OMP THREADPRIVATE(XGFLUX_STRLROOF)
REAL, POINTER, DIMENSION(:)   :: XGFLUX_BLT=>NULL()
!$OMP THREADPRIVATE(XGFLUX_BLT)
REAL, POINTER, DIMENSION(:)   :: XLE_ROOF=>NULL()
!$OMP THREADPRIVATE(XLE_ROOF)
REAL, POINTER, DIMENSION(:)   :: XLE_ROAD=>NULL()
!$OMP THREADPRIVATE(XLE_ROAD)
REAL, POINTER, DIMENSION(:)   :: XLE_GARDEN=>NULL()
!$OMP THREADPRIVATE(XLE_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XLE_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XLE_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XLE_STRLROOF=>NULL()
!$OMP THREADPRIVATE(XLE_STRLROOF)
REAL, POINTER, DIMENSION(:)   :: XLE_BLT=>NULL()
!$OMP THREADPRIVATE(XLE_BLT)
!
REAL, POINTER, DIMENSION(:)   :: XRUNOFF_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XRUNOFF_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XDRAIN_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XDRAIN_GREENROOF)
!
REAL, POINTER, DIMENSION(:)   :: XABS_SW_ROOF=>NULL()
!$OMP THREADPRIVATE(XABS_SW_ROOF)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_SNOW_ROOF=>NULL()
!$OMP THREADPRIVATE(XABS_SW_SNOW_ROOF)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_ROOF=>NULL()
!$OMP THREADPRIVATE(XABS_LW_ROOF)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_SNOW_ROOF=>NULL()
!$OMP THREADPRIVATE(XABS_LW_SNOW_ROOF)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_ROAD=>NULL()
!$OMP THREADPRIVATE(XABS_SW_ROAD)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_SNOW_ROAD=>NULL()
!$OMP THREADPRIVATE(XABS_SW_SNOW_ROAD)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_ROAD=>NULL()
!$OMP THREADPRIVATE(XABS_LW_ROAD)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_SNOW_ROAD=>NULL()
!$OMP THREADPRIVATE(XABS_LW_SNOW_ROAD)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_WALL_A=>NULL()
!$OMP THREADPRIVATE(XABS_SW_WALL_A)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_WALL_B=>NULL()
!$OMP THREADPRIVATE(XABS_SW_WALL_B)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_WALL_A=>NULL()
!$OMP THREADPRIVATE(XABS_LW_WALL_A)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_WALL_B=>NULL()
!$OMP THREADPRIVATE(XABS_LW_WALL_B)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_GARDEN=>NULL()
!$OMP THREADPRIVATE(XABS_SW_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_GARDEN=>NULL()
!$OMP THREADPRIVATE(XABS_LW_GARDEN)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XABS_SW_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_GREENROOF=>NULL()
!$OMP THREADPRIVATE(XABS_LW_GREENROOF)
REAL, POINTER, DIMENSION(:)   :: XG_GREENROOF_ROOF=>NULL()
!$OMP THREADPRIVATE(XG_GREENROOF_ROOF)
REAL, POINTER, DIMENSION(:)   :: XH_BLD_COOL=>NULL()
!$OMP THREADPRIVATE(XH_BLD_COOL)
REAL, POINTER, DIMENSION(:)   :: XT_BLD_COOL=>NULL()
!$OMP THREADPRIVATE(XT_BLD_COOL)
REAL, POINTER, DIMENSION(:)   :: XH_BLD_HEAT=>NULL()
!$OMP THREADPRIVATE(XH_BLD_HEAT)
REAL, POINTER, DIMENSION(:)   :: XLE_BLD_COOL=>NULL()
!$OMP THREADPRIVATE(XLE_BLD_COOL)
REAL, POINTER, DIMENSION(:)   :: XLE_BLD_HEAT=>NULL()
!$OMP THREADPRIVATE(XLE_BLD_HEAT)
REAL, POINTER, DIMENSION(:)   :: XH_WASTE=>NULL()
!$OMP THREADPRIVATE(XH_WASTE)
REAL, POINTER, DIMENSION(:)   :: XLE_WASTE=>NULL()
!$OMP THREADPRIVATE(XLE_WASTE)
REAL, POINTER, DIMENSION(:)   :: XHVAC_COOL=>NULL()
!$OMP THREADPRIVATE(XHVAC_COOL)
REAL, POINTER, DIMENSION(:)   :: XHVAC_HEAT=>NULL()
!$OMP THREADPRIVATE(XHVAC_HEAT)
REAL, POINTER, DIMENSION(:)   :: XCAP_SYS=>NULL()
!$OMP THREADPRIVATE(XCAP_SYS)
REAL, POINTER, DIMENSION(:)   :: XM_SYS=>NULL()
!$OMP THREADPRIVATE(XM_SYS)
REAL, POINTER, DIMENSION(:)   :: XCOP=>NULL()
!$OMP THREADPRIVATE(XCOP)
REAL, POINTER, DIMENSION(:)   :: XQ_SYS=>NULL()
!$OMP THREADPRIVATE(XQ_SYS)
REAL, POINTER, DIMENSION(:)   :: XT_SYS=>NULL()
!$OMP THREADPRIVATE(XT_SYS)
REAL, POINTER, DIMENSION(:)   :: XTR_SW_WIN=>NULL()
!$OMP THREADPRIVATE(XTR_SW_WIN)
REAL, POINTER, DIMENSION(:)   :: XFAN_POWER=>NULL()
!$OMP THREADPRIVATE(XFAN_POWER)
REAL, POINTER, DIMENSION(:)   :: XABS_SW_WIN=>NULL()
!$OMP THREADPRIVATE(XABS_SW_WIN)
REAL, POINTER, DIMENSION(:)   :: XABS_LW_WIN=>NULL()
!$OMP THREADPRIVATE(XABS_LW_WIN)
REAL, POINTER, DIMENSION(:)   :: XEMIT_LW_FAC=>NULL()
!$OMP THREADPRIVATE(XEMIT_LW_FAC)
REAL, POINTER, DIMENSION(:)   :: XEMIT_LW_GRND=>NULL()
!$OMP THREADPRIVATE(XEMIT_LW_GRND)
REAL, POINTER, DIMENSION(:)   :: XT_RAD_IND=>NULL()
!$OMP THREADPRIVATE(XT_RAD_IND)
REAL, POINTER, DIMENSION(:)   :: XREF_SW_GRND=>NULL()
!$OMP THREADPRIVATE(XREF_SW_GRND)
REAL, POINTER, DIMENSION(:)   :: XREF_SW_FAC=>NULL()
!$OMP THREADPRIVATE(XREF_SW_FAC)
REAL, POINTER, DIMENSION(:)   :: XUTCI_IN=>NULL()
!$OMP THREADPRIVATE(XUTCI_IN)
REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSUN=>NULL()
!$OMP THREADPRIVATE(XUTCI_OUTSUN)
REAL, POINTER, DIMENSION(:)   :: XUTCI_OUTSHADE=>NULL()
!$OMP THREADPRIVATE(XUTCI_OUTSHADE)
REAL, POINTER, DIMENSION(:)   :: XTRAD_SUN=>NULL()
!$OMP THREADPRIVATE(XTRAD_SUN)
REAL, POINTER, DIMENSION(:)   :: XTRAD_SHADE=>NULL()
!$OMP THREADPRIVATE(XTRAD_SHADE)
REAL, POINTER, DIMENSION(:)   :: XHU_BLD=>NULL()
!$OMP THREADPRIVATE(XHU_BLD)

CONTAINS

SUBROUTINE DIAG_MISC_TEB_OPTIONS_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_OPTIONS_GOTO_MODEL',0,ZHOOK_HANDLE)
DIAG_MISC_TEB_OPTIONS_MODEL(KFROM)%XUTCI_IN=>XUTCI_IN
DIAG_MISC_TEB_OPTIONS_MODEL(KFROM)%XUTCI_OUTSUN=>XUTCI_OUTSUN
DIAG_MISC_TEB_OPTIONS_MODEL(KFROM)%XUTCI_OUTSHADE=>XUTCI_OUTSHADE
DIAG_MISC_TEB_OPTIONS_MODEL(KFROM)%XTRAD_SUN=>XTRAD_SUN
DIAG_MISC_TEB_OPTIONS_MODEL(KFROM)%XTRAD_SHADE=>XTRAD_SHADE
!
LSURF_MISC_BUDGET=>DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%LSURF_MISC_BUDGET
LSURF_EVAP_BUDGET=>DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%LSURF_EVAP_BUDGET
LSURF_DIAG_ALBEDO=>DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%LSURF_DIAG_ALBEDO
XUTCI_IN => DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%XUTCI_IN
XUTCI_OUTSUN => DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%XUTCI_OUTSUN
XUTCI_OUTSHADE => DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%XUTCI_OUTSHADE
XTRAD_SUN => DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%XTRAD_SUN
XTRAD_SHADE => DIAG_MISC_TEB_OPTIONS_MODEL(KTO)%XTRAD_SHADE
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_OPTIONS_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_MISC_TEB_OPTIONS_GOTO_MODEL

SUBROUTINE DIAG_MISC_TEB_OPTIONS_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_OPTIONS_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_MISC_TEB_OPTIONS_MODEL(KMODEL))
DIAG_MISC_TEB_OPTIONS_MODEL(:)%LSURF_MISC_BUDGET=.FALSE.
DIAG_MISC_TEB_OPTIONS_MODEL(:)%LSURF_EVAP_BUDGET=.FALSE.
DIAG_MISC_TEB_OPTIONS_MODEL(:)%LSURF_DIAG_ALBEDO=.FALSE.
DO J=1,KMODEL
  NULLIFY(DIAG_MISC_TEB_OPTIONS_MODEL(J)%XUTCI_IN)
  NULLIFY(DIAG_MISC_TEB_OPTIONS_MODEL(J)%XUTCI_OUTSUN)
  NULLIFY(DIAG_MISC_TEB_OPTIONS_MODEL(J)%XUTCI_OUTSHADE)
  NULLIFY(DIAG_MISC_TEB_OPTIONS_MODEL(J)%XTRAD_SUN)
  NULLIFY(DIAG_MISC_TEB_OPTIONS_MODEL(J)%XTRAD_SHADE)
END DO
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_OPTIONS_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_TEB_OPTIONS_ALLOC

SUBROUTINE DIAG_MISC_TEB_OPTIONS_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_OPTIONS_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_MISC_TEB_OPTIONS_MODEL)) DEALLOCATE(DIAG_MISC_TEB_OPTIONS_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_OPTIONS_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_TEB_OPTIONS_DEALLO



SUBROUTINE DIAG_MISC_TEB_GOTO_MODEL(KFROM, KTO, LKFROM, KFROM_PATCH, KTO_PATCH)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
INTEGER, INTENT(IN) :: KFROM_PATCH, KTO_PATCH
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XZ0_TOWN=>XZ0_TOWN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XQF_BLD=>XQF_BLD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XFLX_BLD=>XFLX_BLD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XQF_TOWN=>XQF_TOWN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XDQS_TOWN=>XDQS_TOWN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_WALL_A=>XH_WALL_A
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_WALL_B=>XH_WALL_B
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_ROOF=>XH_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_ROAD=>XH_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_GARDEN=>XH_GARDEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_GREENROOF=>XH_GREENROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_STRLROOF=>XH_STRLROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_BLT=>XH_BLT
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_WALL_A=>XRN_WALL_A
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_WALL_B=>XRN_WALL_B
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_ROOF=>XRN_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_ROAD=>XRN_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_GARDEN=>XRN_GARDEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_GREENROOF=>XRN_GREENROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_STRLROOF=>XRN_STRLROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRN_BLT=>XRN_BLT
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_WALL_A=>XGFLUX_WALL_A
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_WALL_B=>XGFLUX_WALL_B
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_ROOF=>XGFLUX_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_ROAD=>XGFLUX_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_GARDEN=>XGFLUX_GARDEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_GREENROOF=>XGFLUX_GREENROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_STRLROOF=>XGFLUX_STRLROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XGFLUX_BLT=>XGFLUX_BLT
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_ROOF=>XLE_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_ROAD=>XLE_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_GARDEN=>XLE_GARDEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_GREENROOF=>XLE_GREENROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_STRLROOF=>XLE_STRLROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_BLT=>XLE_BLT
!
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XRUNOFF_GREENROOF=>XRUNOFF_GREENROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XDRAIN_GREENROOF=>XDRAIN_GREENROOF
!
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_ROOF=>XABS_SW_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_SNOW_ROOF=>XABS_SW_SNOW_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_ROOF=>XABS_LW_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_SNOW_ROOF=>XABS_LW_SNOW_ROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_ROAD=>XABS_SW_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_SNOW_ROAD=>XABS_SW_SNOW_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_ROAD=>XABS_LW_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_SNOW_ROAD=>XABS_LW_SNOW_ROAD
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_WALL_A=>XABS_SW_WALL_A
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_WALL_B=>XABS_SW_WALL_B
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_WALL_A=>XABS_LW_WALL_A
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_WALL_B=>XABS_LW_WALL_B
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_GARDEN=>XABS_SW_GARDEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_GARDEN=>XABS_LW_GARDEN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_GREENROOF=>XABS_SW_GREENROOF
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_GREENROOF=>XABS_LW_GREENROOF
!
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_BLD_COOL=>XH_BLD_COOL
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XT_BLD_COOL=>XT_BLD_COOL
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_BLD_HEAT=>XH_BLD_HEAT
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_BLD_COOL=>XLE_BLD_COOL
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_BLD_HEAT=>XLE_BLD_HEAT
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XHVAC_COOL=>XHVAC_COOL
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XHVAC_HEAT=>XHVAC_HEAT
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XH_WASTE=>XH_WASTE
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XLE_WASTE=>XLE_WASTE
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XCAP_SYS=>XCAP_SYS
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XM_SYS=>XM_SYS
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XCOP=>XCOP
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XQ_SYS=>XQ_SYS
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XT_SYS=>XT_SYS
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XTR_SW_WIN=>XTR_SW_WIN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XFAN_POWER=>XFAN_POWER
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_SW_WIN=>XABS_SW_WIN
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XABS_LW_WIN=>XABS_LW_WIN

DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XEMIT_LW_GRND=>XEMIT_LW_GRND
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XEMIT_LW_FAC=>XEMIT_LW_FAC
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XT_RAD_IND=>XT_RAD_IND
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XREF_SW_GRND=>XREF_SW_GRND
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XREF_SW_FAC=>XREF_SW_FAC
DIAG_MISC_TEB_MODEL(KFROM,KFROM_PATCH)%XHU_BLD=>XHU_BLD
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_GOTO_MODEL',0,ZHOOK_HANDLE)
XZ0_TOWN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XZ0_TOWN
XQF_BLD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XQF_BLD
XFLX_BLD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XFLX_BLD
XQF_TOWN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XQF_TOWN
XDQS_TOWN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XDQS_TOWN
XH_WALL_A=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_WALL_A
XH_WALL_B=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_WALL_B
XH_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_ROOF
XH_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_ROAD
XH_GARDEN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_GARDEN
XH_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_GREENROOF
XH_STRLROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_STRLROOF
XH_BLT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_BLT
XRN_WALL_A=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_WALL_A
XRN_WALL_B=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_WALL_B
XRN_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_ROOF
XRN_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_ROAD
XRN_GARDEN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_GARDEN
XRN_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_GREENROOF
XRN_STRLROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_STRLROOF
XRN_BLT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRN_BLT
XGFLUX_WALL_A=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_WALL_A
XGFLUX_WALL_B=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_WALL_B
XGFLUX_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_ROOF
XGFLUX_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_ROAD
XGFLUX_GARDEN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_GARDEN
XGFLUX_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_GREENROOF
XGFLUX_STRLROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_STRLROOF
XGFLUX_BLT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XGFLUX_BLT
XLE_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_ROOF
XLE_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_ROAD
XLE_GARDEN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_GARDEN
XLE_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_GREENROOF
XLE_STRLROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_STRLROOF
XLE_BLT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_BLT
!
XRUNOFF_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XRUNOFF_GREENROOF
XDRAIN_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XDRAIN_GREENROOF
!
XABS_SW_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_ROOF
XABS_SW_SNOW_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_SNOW_ROOF
XABS_LW_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_ROOF
XABS_LW_SNOW_ROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_SNOW_ROOF
XABS_SW_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_ROAD
XABS_SW_SNOW_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_SNOW_ROAD
XABS_LW_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_ROAD
XABS_LW_SNOW_ROAD=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_SNOW_ROAD
XABS_SW_WALL_A=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_WALL_A
XABS_SW_WALL_B=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_WALL_B
XABS_LW_WALL_A=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_WALL_A
XABS_LW_WALL_B=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_WALL_B
XABS_SW_GARDEN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_GARDEN
XABS_LW_GARDEN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_GARDEN
XABS_SW_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_GREENROOF
XABS_LW_GREENROOF=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_GREENROOF
!
XT_BLD_COOL=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XT_BLD_COOL
XH_BLD_COOL=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_BLD_COOL
XH_BLD_HEAT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_BLD_HEAT
XLE_BLD_COOL=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_BLD_COOL
XLE_BLD_HEAT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_BLD_HEAT
XHVAC_COOL=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XHVAC_COOL
XHVAC_HEAT=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XHVAC_HEAT
XH_WASTE=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XH_WASTE
XLE_WASTE=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XLE_WASTE
XCAP_SYS=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XCAP_SYS
XM_SYS=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XM_SYS
XCOP=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XCOP
XQ_SYS=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XQ_SYS
XT_SYS=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XT_SYS
XTR_SW_WIN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XTR_SW_WIN
XFAN_POWER=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XFAN_POWER
XABS_SW_WIN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_SW_WIN
XABS_LW_WIN=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XABS_LW_WIN

XEMIT_LW_GRND=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XEMIT_LW_GRND
XEMIT_LW_FAC=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XEMIT_LW_FAC
XT_RAD_IND=>DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XT_RAD_IND
XREF_SW_GRND => DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XREF_SW_GRND
XREF_SW_FAC => DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XREF_SW_FAC
XHU_BLD => DIAG_MISC_TEB_MODEL(KTO,KTO_PATCH)%XHU_BLD

IF (LHOOK) CALL DR_HOOK('MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DIAG_MISC_TEB_GOTO_MODEL

SUBROUTINE DIAG_MISC_TEB_ALLOC(KMODEL,KPATCH)
INTEGER, INTENT(IN) :: KMODEL, KPATCH
INTEGER :: J,JP
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DIAG_MISC_TEB_MODEL(KMODEL,KPATCH))
DO J=1,KMODEL
 DO JP=1,KPATCH
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XZ0_TOWN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XQF_BLD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XFLX_BLD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XQF_TOWN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XDQS_TOWN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_WALL_A)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_WALL_B)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_GARDEN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_STRLROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_BLT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_WALL_A)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_WALL_B)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_GARDEN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_STRLROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRN_BLT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_WALL_A)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_WALL_B)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_GARDEN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_STRLROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XGFLUX_BLT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_GARDEN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_STRLROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_BLT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XRUNOFF_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XDRAIN_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_SNOW_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_SNOW_ROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_SNOW_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_SNOW_ROAD)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_WALL_A)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_WALL_B)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_WALL_A)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_WALL_B)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_GARDEN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_GARDEN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_GREENROOF)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XT_BLD_COOL)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_BLD_COOL)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_BLD_HEAT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_BLD_COOL)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_BLD_HEAT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XHVAC_COOL)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XHVAC_HEAT)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XH_WASTE)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XLE_WASTE)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XCAP_SYS)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XM_SYS)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XCOP)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XQ_SYS)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XT_SYS)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XTR_SW_WIN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XFAN_POWER)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_SW_WIN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XABS_LW_WIN)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XEMIT_LW_GRND)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XEMIT_LW_FAC)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XT_RAD_IND)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XREF_SW_GRND)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XREF_SW_FAC)
  NULLIFY(DIAG_MISC_TEB_MODEL(J,JP)%XHU_BLD)
 ENDDO
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_TEB_ALLOC

SUBROUTINE DIAG_MISC_TEB_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DIAG_MISC_TEB_MODEL)) DEALLOCATE(DIAG_MISC_TEB_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DIAG_MISC_TEB_N:DIAG_MISC_TEB_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DIAG_MISC_TEB_DEALLO


END MODULE MODD_DIAG_MISC_TEB_n
