!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!        ###############
         MODULE MODD_SV_n
!        ###############
!
!!****  *MODD_NSV* - declaration of scalar variables numbers
!!
!!    PURPOSE
!!    -------
!!       Arrays to store the per-model NSV_* values number (suffix _A denote an array)
!!
!!    AUTHOR
!!    ------
!!      P. Tulet   Meteo-France
!!
!!    MODIFICATIONS
!!    -------------
!!      Original  01/2004
!!
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SV_t
!
!###############################################################################
!
! variables updated for the current model
!
  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CSV ! name of the scalar variables
  INTEGER    :: NSV_CHSBEG, NSV_CHSEND    !  index of first and last gas chemistry related scalar variable
  INTEGER    :: NBEQ                      ! number of chemical gas species in the surface scheme
  INTEGER    :: NSV_DSTBEG, NSV_DSTEND    ! index of first and last dust related scalar variable
  INTEGER    :: NDSTEQ                    ! number of dust related species in scalar variables list
  INTEGER    :: NSV_SLTBEG, NSV_SLTEND    ! index of first and last sea salt related scalar variable
  INTEGER    :: NSLTEQ                    ! number of sea salt related species in scalar variables list
  INTEGER    :: NSV_AERBEG, NSV_AEREND    ! index of first and last aerosol related scalar variabl
  INTEGER    :: NAEREQ                    ! number of aerosols variables

!
!
END TYPE SV_t

TYPE(SV_t), ALLOCATABLE, TARGET, SAVE :: SV_MODEL(:)

 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CSV=>NULL()
!$OMP THREADPRIVATE(CSV)
INTEGER, POINTER :: NSV_CHSBEG=>NULL(), NSV_CHSEND=>NULL()
!$OMP THREADPRIVATE(NSV_CHSBEG, NSV_CHSEND)
INTEGER, POINTER :: NBEQ=>NULL()
!$OMP THREADPRIVATE(NBEQ)
INTEGER, POINTER :: NSV_DSTBEG=>NULL(), NSV_DSTEND=>NULL()
!$OMP THREADPRIVATE(NSV_DSTBEG, NSV_DSTEND)
INTEGER, POINTER :: NDSTEQ=>NULL()
!$OMP THREADPRIVATE(NDSTEQ)
INTEGER, POINTER :: NSV_SLTBEG=>NULL(), NSV_SLTEND=>NULL()
!$OMP THREADPRIVATE(NSV_SLTBEG, NSV_SLTEND)
INTEGER, POINTER :: NSLTEQ=>NULL()
!$OMP THREADPRIVATE(NSLTEQ)
INTEGER, POINTER :: NSV_AERBEG=>NULL(), NSV_AEREND=>NULL()
!$OMP THREADPRIVATE(NSV_AERBEG, NSV_AEREND)
INTEGER, POINTER :: NAEREQ=>NULL()
!$OMP THREADPRIVATE(NAEREQ)

CONTAINS

SUBROUTINE SV_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
SV_MODEL(KFROM)%CSV=>CSV
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_SV_N:SV_GOTO_MODEL',0,ZHOOK_HANDLE)
CSV=>SV_MODEL(KTO)%CSV
NSV_CHSBEG=>SV_MODEL(KTO)%NSV_CHSBEG
NSV_CHSEND=>SV_MODEL(KTO)%NSV_CHSEND
NBEQ=>SV_MODEL(KTO)%NBEQ
NSV_DSTBEG=>SV_MODEL(KTO)%NSV_DSTBEG
NSV_DSTEND=>SV_MODEL(KTO)%NSV_DSTEND
NDSTEQ=>SV_MODEL(KTO)%NDSTEQ
NSV_SLTBEG=>SV_MODEL(KTO)%NSV_SLTBEG
NSV_SLTEND=>SV_MODEL(KTO)%NSV_SLTEND
NSLTEQ=>SV_MODEL(KTO)%NSLTEQ
NSV_AERBEG=>SV_MODEL(KTO)%NSV_AERBEG
NSV_AEREND=>SV_MODEL(KTO)%NSV_AEREND
NAEREQ=>SV_MODEL(KTO)%NAEREQ
IF (LHOOK) CALL DR_HOOK('MODD_SV_N:SV_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE SV_GOTO_MODEL

SUBROUTINE SV_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SV_N:SV_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(SV_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(SV_MODEL(J)%CSV)
ENDDO
SV_MODEL(:)%NBEQ=0
SV_MODEL(:)%NSV_CHSBEG=0
SV_MODEL(:)%NSV_CHSEND=0
SV_MODEL(:)%NSV_DSTBEG=0
SV_MODEL(:)%NSV_DSTEND=0
SV_MODEL(:)%NDSTEQ=0
SV_MODEL(:)%NSV_SLTBEG=0
SV_MODEL(:)%NSV_SLTEND=0
SV_MODEL(:)%NSLTEQ=0
SV_MODEL(:)%NSV_AERBEG=0
SV_MODEL(:)%NSV_AEREND=0
SV_MODEL(:)%NAEREQ=0
IF (LHOOK) CALL DR_HOOK("MODD_SV_N:SV_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE SV_ALLOC

SUBROUTINE SV_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SV_N:SV_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(SV_MODEL)) DEALLOCATE(SV_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_SV_N:SV_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE SV_DEALLO

END MODULE MODD_SV_n
