!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     #########
      SUBROUTINE WRITE_DIAG_SEB_WATFLUX_n(HPROGRAM)
!     #################################
!
!!****  *WRITE_DIAG_SEB_WATFLUX_n* - writes WATFLUX diagnostics
!!
!!    PURPOSE
!!    -------
!!
!!
!!**  METHOD
!!    ------
!!          
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!	V. Masson   *Meteo France*	
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    01/2004
!!      Modified    01/2006 : sea flux parameterization.
!!      S.Bielli    11/2012 : write HU2M_WAT mis placed
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
USE MODD_SURF_PAR,      ONLY : XUNDEF
!
USE MODD_DIAG_SURF_ATM_n,ONLY : LPROVAR_TO_DIAG, LRESET_BUDGETC
!
USE MODD_DIAG_WATFLUX_n,ONLY : N2M, LRAD_BUDGET, LSURF_BUDGET, LCOEF,    &
                                 LSURF_VARS,                               &
                                 XRN, XH, XLE, XLEI, XGFLUX,               &
                                 XRI, XCD, XCH, XCE, XZ0, XZ0H,            &
                                 XT2M, XQ2M, XHU2M, XT2M_MIN, XT2M_MAX,    &
                                 XZON10M, XMER10M, XQS, XDIAG_TS,          &
                                 XSWD, XSWU, XLWD, XLWU, XSWBD, XSWBU,     &
                                 XFMU, XFMV, LSURF_BUDGETC,                &
                                 XRNC, XHC, XLEC, XGFLUXC, XSWDC, XSWUC,   &
                                 XLWDC, XLWUC, XFMUC, XFMVC, XLEIC,        &
                                 XHU2M_MIN, XHU2M_MAX, XWIND10M, XWIND10M_MAX  
!
USE MODD_WATFLUX_n,ONLY : LINTERPOL_TS
!
USE MODD_CH_WATFLUX_n,  ONLY : XDEP, CCH_DRY_DEP, CCH_NAMES, NBEQ
!
USE MODI_INIT_IO_SURF_n
USE MODI_WRITE_SURF
USE MODI_END_IO_SURF_n
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*       0.1   Declarations of arguments
!              -------------------------
!
 CHARACTER(LEN=6),  INTENT(IN)  :: HPROGRAM ! program calling
!
!*       0.2   Declarations of local variables
!              -------------------------------
!
INTEGER           :: IRESP          ! IRESP  : return-code if a problem appears
 CHARACTER(LEN=12) :: YRECFM         ! Name of the article to be read
 CHARACTER(LEN=100):: YCOMMENT       ! Comment string
 CHARACTER(LEN=2)  :: YNUM
!
INTEGER           :: JSV, JSW
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-------------------------------------------------------------------------------
!
!         Initialisation for IO
!
IF (LHOOK) CALL DR_HOOK('WRITE_DIAG_SEB_WATFLUX_N',0,ZHOOK_HANDLE)
 CALL INIT_IO_SURF_n(HPROGRAM,'WATER ','WATFLX','WRITE')
!
!
!*       2.     Richardson number :
!               -----------------
!
IF (N2M>=1) THEN

YRECFM='RI_WAT'
YCOMMENT='X_Y_'//YRECFM
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XRI(:),IRESP,HCOMMENT=YCOMMENT)
!
END IF
!
!*       3.     Energy fluxes :
!               -------------
!
IF (LSURF_BUDGET) THEN

YRECFM='RN_WAT'
YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XRN(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='H_WAT'
YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XH(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='LE_WAT'
YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XLE(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='LEI_WAT'
YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XLEI(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='GFLUX_WAT'
YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XGFLUX(:),IRESP,HCOMMENT=YCOMMENT)
!
IF (LRAD_BUDGET) THEN
!       
   YRECFM='SWD_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XSWD(:),IRESP,HCOMMENT=YCOMMENT)
   !
   YRECFM='SWU_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XSWU(:),IRESP,HCOMMENT=YCOMMENT)
   !
   YRECFM='LWD_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XLWD(:),IRESP,HCOMMENT=YCOMMENT)
   !
   YRECFM='LWU_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XLWU(:),IRESP,HCOMMENT=YCOMMENT)
   !
   DO JSW=1, SIZE(XSWBD,2)
      YNUM=ACHAR(48+JSW)
      !
      YRECFM='SWD_WAT_'//YNUM
      YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
      !
      CALL WRITE_SURF(HPROGRAM,YRECFM,XSWBD(:,JSW),IRESP,HCOMMENT=YCOMMENT)
      !
      YRECFM='SWU_WAT_'//YNUM
      YCOMMENT='X_Y_'//YRECFM//' (W/m2)'
      !
      CALL WRITE_SURF(HPROGRAM,YRECFM,XSWBU(:,JSW),IRESP,HCOMMENT=YCOMMENT)
      !
   ENDDO
!
ENDIF
!
YRECFM='FMU_WAT'
YCOMMENT='X_Y_'//YRECFM//' (kg/ms2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XFMU(:),IRESP,HCOMMENT=YCOMMENT)
YRECFM='FMV_WAT'
YCOMMENT='X_Y_'//YRECFM//' (kg/ms2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XFMV(:),IRESP,HCOMMENT=YCOMMENT)
!
END IF
!
IF (LSURF_BUDGETC) THEN

YRECFM='RNC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XRNC(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='HC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XHC(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='LEC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XLEC(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='LEIC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XLEIC(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='GFLUXC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XGFLUXC(:),IRESP,HCOMMENT=YCOMMENT)
!
IF (LRAD_BUDGET .OR. (LSURF_BUDGETC .AND. .NOT.LRESET_BUDGETC)) THEN
!
   YRECFM='SWDC_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XSWDC(:),IRESP,HCOMMENT=YCOMMENT)
   !
   YRECFM='SWUC_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XSWUC(:),IRESP,HCOMMENT=YCOMMENT)
   !
   YRECFM='LWDC_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XLWDC(:),IRESP,HCOMMENT=YCOMMENT)
   !
   YRECFM='LWUC_WAT'
   YCOMMENT='X_Y_'//YRECFM//' (J/m2)'
   !
   CALL WRITE_SURF(HPROGRAM,YRECFM,XLWUC(:),IRESP,HCOMMENT=YCOMMENT)
!
ENDIF
!
YRECFM='FMUC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (kg/ms)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XFMUC(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='FMVC_WAT'
YCOMMENT='X_Y_'//YRECFM//' (kg/ms)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XFMVC(:),IRESP,HCOMMENT=YCOMMENT)
!
END IF
!
!
!*       4.     Transfer coefficients
!               ---------------------
!
IF (LCOEF) THEN

YRECFM='CD_WAT'
YCOMMENT='X_Y_'//YRECFM
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XCD(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='CH_WAT'
YCOMMENT='X_Y_'//YRECFM
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XCH(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='CE_WAT'
YCOMMENT='X_Y_'//YRECFM
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XCE(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='Z0_WAT'
YCOMMENT='X_Y_'//YRECFM
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XZ0(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='Z0H_WAT'
YCOMMENT='X_Y_'//YRECFM
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XZ0H(:),IRESP,HCOMMENT=YCOMMENT)
!
END IF
!
!
!*       5.     Surface humidity
!               ----------------
!
IF (LSURF_VARS) THEN

YRECFM='QS_WAT'
YCOMMENT='X_Y_'//YRECFM//' (KG/KG)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XQS(:),IRESP,HCOMMENT=YCOMMENT)
!
ENDIF
!

!
!*       6.     parameters at 2 and 10 meters :
!               -----------------------------
!
IF (N2M>=1) THEN
!
YRECFM='T2M_WAT'
YCOMMENT='X_Y_'//YRECFM//' (K)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XT2M(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='T2MMIN_WAT'
YCOMMENT='X_Y_'//YRECFM//' (K)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XT2M_MIN(:),IRESP,HCOMMENT=YCOMMENT)
XT2M_MIN(:)=XUNDEF
!
YRECFM='T2MMAX_WAT'
YCOMMENT='X_Y_'//YRECFM//' (K)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XT2M_MAX(:),IRESP,HCOMMENT=YCOMMENT)
XT2M_MAX(:)=0.0
!
YRECFM='Q2M_WAT'
YCOMMENT='X_Y_'//YRECFM//' (KG/KG)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XQ2M(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='HU2M_WAT'
YCOMMENT='X_Y_'//YRECFM//' (-)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XHU2M(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='HU2MMIN_WAT'
YCOMMENT='X_Y_'//YRECFM//' (-)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XHU2M_MIN(:),IRESP,HCOMMENT=YCOMMENT)
XHU2M_MIN(:)=XUNDEF
!
YRECFM='HU2MMAX_WAT'
YCOMMENT='X_Y_'//YRECFM//' (-)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XHU2M_MAX(:),IRESP,HCOMMENT=YCOMMENT)
XHU2M_MAX(:)=-XUNDEF
!
YRECFM='ZON10M_WAT'
YCOMMENT='X_Y_'//YRECFM//' (M/S)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XZON10M(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='MER10M_WAT'
YCOMMENT='X_Y_'//YRECFM//' (M/S)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XMER10M(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='W10M_WAT'
YCOMMENT='X_Y_'//YRECFM//' (M/S)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XWIND10M(:),IRESP,HCOMMENT=YCOMMENT)
!
YRECFM='W10MMAX_WAT'
YCOMMENT='X_Y_'//YRECFM//' (M/S)'
!
 CALL WRITE_SURF(HPROGRAM,YRECFM,XWIND10M_MAX(:),IRESP,HCOMMENT=YCOMMENT)
XWIND10M_MAX(:)=0.0
!
END IF
!
!
!*       7.     chemical diagnostics:
!               --------------------
!
IF (NBEQ>0 .AND. CCH_DRY_DEP=="WES89 ") THEN
  DO JSV = 1,SIZE(CCH_NAMES,1)
    YRECFM='DV_WAT_'//TRIM(CCH_NAMES(JSV))
    WRITE(YCOMMENT,'(A13,I3.3)')'(m/s) DV_WAT_',JSV
    CALL WRITE_SURF(HPROGRAM,YRECFM,XDEP(:,JSV),IRESP,HCOMMENT=YCOMMENT)
  END DO
ENDIF
!
!
!*       8.     prognostic variable diagnostics:
!               --------------------------------
!
IF(LPROVAR_TO_DIAG.OR.LINTERPOL_TS)THEN
!
  YRECFM='TS_WATER'
  YCOMMENT='TS_WATER (K)'
  CALL WRITE_SURF(HPROGRAM,YRECFM,XDIAG_TS(:),IRESP,HCOMMENT=YCOMMENT)
!
ENDIF
!
!-------------------------------------------------------------------------------
!
!         End of IO
!
 CALL END_IO_SURF_n(HPROGRAM)
IF (LHOOK) CALL DR_HOOK('WRITE_DIAG_SEB_WATFLUX_N',1,ZHOOK_HANDLE)
!
!
END SUBROUTINE WRITE_DIAG_SEB_WATFLUX_n
